package MovaMotion;

use strict;

use base qw( MT::App );

use MT;
use MT::Blog;
use MT::Session;
use MT::Author;
use MT::Entry;
use MT::Tag;
use File::Spec;
use Image::Size;
use MT::Util qw( perl_sha1_digest_hex is_valid_url encode_url );
use MT::I18N qw( encode_text guess_encoding );

@MovaMotion::ISA = qw( MT::App );

our $original_charset;

sub init_request {
    my $app = shift;
    $app->SUPER::init_request( @_ );
    unless ( $app->param( 'redirect' ) ) {
        if ( $app->param( 'type' ) eq 'quote' ) {
            if ( $app->param( 'title' ) ) {
                if ( $app->config( 'PublishCharset' ) ne 'UTF-8' ) {
                    $original_charset = $app->config( 'PublishCharset' );
                    my $query_string = $app->query_string;
                    $query_string =~ s/(title=.*$)/encode_url( encode_text( $1, 'UTF-8', $original_charset ) )/e;
                    $app->redirect( $app->base . $app->uri . '?' . $query_string . ';redirect=1' );
                }
            }
        }
    }
    $app->{ default_mode } = 'view';
    $app->add_methods( view => \&_view );
    $app->add_methods( login => \&_login );
    $app->add_methods( logout => \&_logout );
    $app->add_methods( redirect => \&_redirect );
    my $ua = $app->get_header( 'User-Agent' );
    $app->{ requires_login } = 1 if ( $ua =~ /Mozilla/ );
    $app;
}

sub _view {
    my $app = shift;
    my $title = $app->param( 'title' );
    if ( $original_charset ) {
        $app->config( 'PublishCharset', $original_charset );
        $title = encode_text( $title, 'UTF-8', $original_charset );
    }
    my $ua = $app->get_header( 'User-Agent' );
    my %param; my $tmpl;
    my $plugin = MT->component( 'MovaMotion' );
    $app->config( 'TemplatePath', File::Spec->catdir( $plugin->path, 'tmpl' ) );
    my $blog_id  = $app->param( 'blog_id' );
    my $offset = $app->param( 'offset' ) || 0;
    my $limit  = $app->param( 'limit' ) || 20;
    my $tag = lc ( $app->param( 'reply' ) );
    my $reply2 = $app->param( 'reply2' );
    my $author = $app->param( 'author' );
    my $mailpost = $app->param( 'mailpost' );
    $blog_id = undef if ( $blog_id !~ /^[0-9]{1,}$/ );
    $offset = 0 if ( $offset !~ /^[0-9]{1,}$/ );
    $limit = 20 if ( $limit !~ /^[0-9]{1,}$/ );
    %param->{ offset } = $offset;
    %param->{ limit } = $limit;
    %param->{ mailpost } = $mailpost;
    my $sessid = $app->param( 'sessid' );
    my $type = $app->param( 'type' );
    my $popup = $app->param( 'popup' );
    %param->{ type } = $type;
    my $user;
    if ( $ua =~ /Mozilla/ ) {
        $user = $app->user;
        $sessid = '';
    } else {
        $user = _get_auther( $app, $sessid );
    }
    my $blog = $app->blog;
    unless ( defined $blog ) {
        if ( MT::Blog->count == 1 ) {
            $blog = MT::Blog->load( undef, { limit => 1 } );
            $blog_id = $blog->id;
        }
    }
    %param->{ blog_id } = $blog_id;
    %param->{ reply } = $app->param( 'reply' );
    if ( (! defined $user ) || (! $blog_id ) ) {
        $tmpl = 'login.tmpl';
        return $app->build_page( $tmpl, \%param );
    } else {
        unless ( _can_access ( $user, $blog_id ) ) {
            return $plugin->translate( 'Permission denied' );
        }
        %param->{ username } = $user->name;
        %param->{ user_id } = $user->id;
        %param->{ user_setting_url } = $app->base . $app->path . $app->config( 'AdminScript' ) . '?__mode=view&_type=author&id=' . $user->id;
        if ( $type eq 'set_uid' ) {
            my $return_url = $app->param( 'return_url' );
            $return_url =~ s/set_uid=[a-z]{1,}//;
            my $mobile_id = _get_mobile_id( $app );
            if ( $mobile_id ) {
                $user->mobile_id( $mobile_id );
                $user->save or return $user->errstr;
                $return_url .= ';set_uid=successful';
            } else {
                $return_url .= ';set_uid=failed';
            }
            $app->redirect( $return_url );
        }
        if ( _can_post ( $user, $blog_id ) ) {
            if ( $type eq 'delete' ) {
                my $entry = MT::Entry->load( $app->param( 'id' ) );
                if ( _can_delete( $user, $blog_id, $entry ) ) {
                    $entry->remove or die $entry->errstr;
                    %param->{ save_delete } = 1;
                }
            }
            if ( ( $type eq 'photo' ) || ( $type eq 'quote' ) ) {
                %param->{ photo } = 1;
                %param->{ p_reply2 } = $app->param( 'p_reply2' );
                my $postaddress = $plugin->get_config_value( 'movamotion_address', 'blog:'. $blog_id );
                my $token = $user->mail_token;
                if ( MT::Plugin::MovaMotion::_hdlr_if_au() ) {
                    %param->{ token_au } = $token;
                } else {
                    # %param->{ token_au } = $token;
                    $postaddress =~ s/@/+$token@/;
                }
                %param->{ postaddress } = $postaddress;
                if ( $type eq 'quote' ) {
                    my $cite = $app->param( 'cite' );
                    if ( is_valid_url ( $cite ) ) {
                        %param->{ popup } = 1;
                        %param->{ cite } = $cite;
                        if ( $title ) {
                            %param->{ cite_title } = $title;
                        } else {
                            use LWP::UserAgent;
                            use HTTP::Request::Common;
                            my $request  = GET( $cite );
                            my $ua = LWP::UserAgent->new( agent => 'Mozilla/5.0 (MovaMotion 1.0 Beta X_FORWARDED_FOR:' . $app->remote_ip . ')' );
                            my $res = $ua->request( GET $cite );
                            if ( $res->is_success ) {
                                my $content_type = $res->header( 'Content-Type' );
                                my $content_length = $res->header( 'Content-Length' );
                                if ( ( $content_type =~ m!^text/! ) || ( $content_type =~ m!^application/xhtml\+xml! ) ) {
                                    if ( $content_length < 1024000 ) {
                                        my $document = $res->content;
                                        if ( $document ) {
                                            my $encoding = guess_encoding( $document );
                                            if ( $document =~ /<meta\s*http\-equiv\s*=\s*"content\-type"\s*content=".*?charset=(.*?)"/i ) {
                                                $encoding = $1;
                                            }
                                            $document = encode_text( $document, $encoding, undef );
                                            if ( $document =~ m/<title.*?>(.*?)</si ) {
                                                %param->{ cite_title } = $1;
                                            }
                                        }
                                    }
                                    
                                } elsif ( $content_type =~ m!^image/! ) {
                                    %param->{ cite_title } = $plugin->translate( 'Photo' ) . ':';
                                }
                                unless ( %param->{ cite_title } ) {
                                    %param->{ cite_title } = $plugin->translate( 'Quote' ) . ':';
                                }
                            }
                        }
                    }
                }
            }
            if ( $type eq 'mailpost' ) {
                my $res = MT::Plugin::MovaMotion::_post_from_email( $blog, $user->id );
                if ( $res ) {
                    my $return_url = $app->base . $app->uri( mode => 'view',
                                                             args => { blog_id => $blog_id,
                                                                       sessid => $sessid,
                                                                       do_post => 1, } );
                    return $app->redirect( $return_url );
                }
            }
            if ( $type eq 'entry' ) {
                %param->{ entry } = 1;
            }
            if ( ( $type eq 'post' ) || ( $type eq 'photo_pc' ) ) {
                my $title = $app->param( 'title' );
                if ( $title ) {
                    my $entry = MT::Entry->new;
                    $entry->blog_id( $blog_id );
                    $entry->status( MT::Entry::RELEASE() );
                    $entry->author_id( $user->id );
                    $entry->title( $title );
                    $entry->text( $app->param( 'text' ) );
                    $entry->text_more( $app->param( 'text_more' ) );
                    $entry->excerpt( $app->param( 'excerpt' ) );
                    $entry->keywords( $app->param( 'keywords' ) );
                    my $cite = $app->param( 'cite' );
                    if ( $cite ) {
                        if ( is_valid_url( $cite ) ) {
                            $entry->cite( $cite );
                        }
                    }
                    my $quote = $app->param( 'quote' );
                    if ( $quote ) {
                        $entry->quote( $quote );
                    }
                    if ( $type eq 'photo_pc' ) {
                        my $fmgr = $blog->file_mgr;
                        my $q = $app->param;
                        my $file = $q->upload( 'file' );
                        my @suffix = split( /\./, $file );
                        my $file_ext = lc ( pop( @suffix ) );
                        if ( ( $file_ext =~ /^jpe?g$/i ) || ( $file_ext =~ /^gif$/i ) || ( $file_ext =~ /^png$/i ) ) {
                            my $site_path = MT::Plugin::MovaMotion::_site_path( $blog );
                            my $site_url = MT::Plugin::MovaMotion::_site_url( $blog );
                            my $outdir = $plugin->get_config_value( 'movamotion_outdir', 'blog:'. $blog_id );
                            my $imagewidth = $plugin->get_config_value( 'movamotion_imagewidth', 'blog:'. $blog_id );
                            my $imagemarkup = $plugin->get_config_value( 'movamotion_imagemarkup', 'blog:'. $blog_id );
                            my @imagemarkups = split( /,/, $imagemarkup ) if $imagemarkup;
                            $outdir = File::Spec->catfile ( $blog->site_path, $outdir );
                            unless ( $fmgr->exists( $outdir ) ) {
                                $fmgr->mkpath( $outdir );
                            }
                            my $out = File::Spec->catfile ( $outdir, $file );
                            $out = MT::Plugin::MovaMotion::_uniq_fileneme( $out );
                            my $temp = "$out.new";
                            local *OUT;
                            open ( OUT, ">$out" ) or die "Can't open $out!";
                            binmode ( OUT );
                            while( read ( $file, my $buffer, 1024 ) ) {
                                print OUT $buffer;
                            }
                            close (OUT);
                            $fmgr->rename( $temp, $out );
                            my %asset_elements = ( file => $out,
                                                   blog_id => $blog_id,
                                                   author_id => $user->id, );
                            my $asset = MT::Plugin::MovaMotion::_save_asset( $app, $blog, %asset_elements );
                            my $asset_id = $asset->id;
                            my $asset_url = $asset->url;
                            my $alt = $plugin->translate( 'Photo' );
                            my ( $origw, $origh ) = imgsize( $out );
                            my $link;
                            if ( $origw < $imagewidth ) {
                                $link .= "<img alt=\"$alt\" src=\"$asset_url\" width=\"$origw\" height=\"$origh\" />";
                                if ( $imagemarkup ) {
                                    $link = $imagemarkups[0] . $link . $imagemarkups[1];
                                }
                            } else {
                                my %param = ( Width => $imagewidth );
                                my ( $thumbnail, $w, $h ) = $asset->thumbnail_file( %param );
                                my $regex_path = quotemeta( $site_path );
                                $thumbnail =~ s/^$regex_path/$site_url/;
                                $link = "<a href=\"$asset_url\">";
                                $link .= "<img alt=\"$alt\" src=\"$thumbnail\" width=\"$w\" height=\"$h\" /></a>";
                                if ( $imagemarkup ) {
                                    $link = $imagemarkups[0] . $link . $imagemarkups[1];
                                }
                            }
                            $link = "<form mt:asset-id=\"$asset_id\">" . $link .'</form>';
                            $entry->text_more( $link );
                        }
                    }
                    my @tags;
                    for my $reply ( $title =~ m/@([a-zA-Z0-9]{1,})/g ) {
                        push ( @tags, lc ( $reply ) );
                    }
                    $entry->set_tags ( @tags ) if ( @tags );
                    $entry->save
                        or die $entry->errstr;
                    if ( _can_rebuild ( $user, $blog_id ) ) {
                        MT::Util::start_background_task(
                            sub { $app->rebuild_entry( Entry => $entry->id,
                                                       BuildDependencies => 1 );
                            }
                        );
                    }
                    my $return_url = $app->base . $app->uri( mode => 'view',
                                                             args => { blog_id => $blog_id,
                                                                       sessid => $sessid,
                                                                       popup => $popup } );
                    return $app->redirect( $return_url );
                }
            }
            %param->{ can_post } = 1;
        }
    }
    my $last = $offset + $limit;
    my $terms = { blog_id => $blog_id, status => MT::Entry::RELEASE() };
    my $count;
    if ( $tag ) {
        my $obj = MT::Tag->load( { name => $tag } );
        $count = MT::Entry->tagged_count( $obj->id, $terms ) if defined $obj;
    } else {
        if ( $author ) {
            my $obj;
            if ( $user->name eq $author ) {
                $obj = $user;
                %param->{ my_archive } = 1;
            } else {
                $obj = MT::Author->load( { name => $author } );
            }
            if ( defined $obj ) {
                $terms->{ author_id } = $obj->id;
            } else {
                $author = undef;
            }
        }
        $count = MT::Entry->count( $terms );
    }
    if ( $last < $count ) {
        %param->{ next } = 1;
        %param->{ nextoffset } = $last;
    }
    if ( $offset ) {
        %param->{ prev } = 1;
        my $prev = $last - $limit * 2;
        $prev = 0 if ( $prev < 0 );
        %param->{ prevoffset } = $prev;
    }
    my $return_url = $app->base . $app->uri . '?' . $app->query_string;
    %param->{ return_url } = $return_url;
    %param->{ sessid } = $sessid;
    my $set_uid = $app->param( 'set_uid' );
    if ( $set_uid ) {
        %param->{ status_message } = 1;
        %param->{ set_uid } = $set_uid;
    }
    if ( $popup ) {
        %param->{ popup } = 1;
    }
    %param->{ tag } = $tag;
    %param->{ author } = $author;
    %param->{ reply2 } = $reply2;
    if ( $type eq 'settings' ) {
        %param->{ settings } = 1;
    }
    $tmpl = 'view.tmpl';
    return $app->build_page( $tmpl, \%param );
}

sub _login {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    my $blog_id  = $app->param( 'blog_id' );
    if ( $ua =~ /Mozilla/ ) {
        if ( $app->user ) {
            my $return_url = $app->base . $app->uri( mode => 'view',
                                                     args => { blog_id => $blog_id } );
            return $app->redirect( $return_url );
        }
    }
    my $plugin = MT->component( 'MovaMotion' );
    my $type = $app->param( 'type' );
    my $author;
    if ( $type eq 'easy' ) {
        my $mobile_id = _get_mobile_id( $app );
        my @authors = MT::Author->search_by_meta( mobile_id => $mobile_id );
        return $plugin->translate( 'Unknown user' ) unless @authors;
        $author = $authors[0];
    } else {
        my $username = $app->param( 'username' );
        my $password = $app->param( 'password' );
        $author = MT::Author->load( { name => $username } );
        return $plugin->translate( 'Unknown user' ) unless defined $author;
        return $plugin->translate( 'Invalid password' ) unless $author->is_valid_password( $password );
    }
    my $sess = MT::Session->get_by_key ( { email => $author->email,
                                           name  => $author->name,
                                           kind  => 'SI' } );
    $sess->start( time );
    unless ( $sess->id ) {
        $sess->id ( $app->make_magic_token );
    }
    $sess->save or return 'Error saving session';
    my $return_url = $app->base . $app->uri( mode => 'view',
                                             args => { blog_id => $blog_id, sessid => $sess->id } );
    return $app->redirect( $return_url );
}

sub _logout {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    if ( $ua =~ /Mozilla/ ) {
        $app->logout();
    }
    my $sessid = $app->param( 'sessid' );
    my $sess = MT::Session->load ( { id => $sessid,
                                     kind  => 'SI' } );
    if ( defined $sess ) {
        $sess->remove or die $sess->errstr;
    }
    my $blog_id  = $app->param( 'blog_id' );
    $blog_id = undef if ( $blog_id !~ /^[0-9]{1,}$/ );
    my $return_url = $app->base . $app->uri( mode => 'view',
                                             args => { blog_id => $blog_id, logout => 1 } );
    return $app->redirect( $return_url );
}

sub _redirect {
    my $app = shift;
    my $plugin = MT->component( 'MovaMotion' );
    $app->config( 'TemplatePath', File::Spec->catdir( $plugin->path, 'tmpl' ) );
    my $url = $app->param( 'url' );
    my %param; my $tmpl;
    %param->{ url } = $url;
    $tmpl = 'redirect.tmpl';
    return $app->build_page( $tmpl, \%param );
}

sub _get_auther {
    my ( $app, $sessid ) = @_;
    my $author;
    my $sess = MT::Session->load ( { id => $sessid,
                                     kind  => 'SI' } );
    if ( defined $sess ) {
       my $sess_timeout = $app->config->UserSessionTimeout;
       if ( ( time - $sess->start ) < $sess_timeout ) {
           $author = MT::Author->load( { name => $sess->name } );
           $sess->start( time );
           $sess->save or die $sess->errstr;
       } else {
            my $blog_id  = $app->param( 'blog_id' );
            $blog_id = undef if ( $blog_id !~ /^[0-9]{1,}$/ );
            my $return_url = $app->base . $app->uri( mode => 'view', args => { blog_id => $blog_id, logout => 1 } );
            $app->redirect( $return_url );
       }
    }
    return $author;
}

sub _can_post {
    my ( $user, $blog_id ) = @_;
    return 0 unless defined $user;
    my $can_post = $user->is_superuser;
    unless ( $can_post ) {
        $can_post = $user->permissions( $blog_id )->can_post;
    }
    return $can_post;
}

sub _can_delete {
    my ( $user, $blog_id, $entry ) = @_;
    return 0 unless defined $user;
    return 0 unless defined $entry;
    return 1 if ( $user->id == $entry->author_id );
    my $can_delete = $user->is_superuser;
    unless ( $can_delete ) {
        $can_delete = $user->permissions( $blog_id )->can_edit_all_posts;
    }
    return $can_delete;
}

sub _can_rebuild {
    my ( $user, $blog_id ) = @_;
    return 0 unless defined $user;
    my $can_rebuild = $user->is_superuser;
    unless ( $can_rebuild ) {
        $can_rebuild = $user->permissions( $blog_id )->can_rebuild;
    }
    return $can_rebuild;
}

sub _can_access {
    my ( $user, $blog_id ) = @_;
    return 0 unless defined $user;
    my $can_access = $user->is_superuser;
    unless ( $can_access ) {
        $can_access = MT::Permission->load( { blog_id => $blog_id, author_id => $user } );
    }
    return $can_access;
}

sub _get_mobile_id {
    my $app = shift;
    my $mobile_id;
    my $user_agent = $app->get_header( 'User-Agent' );
    my @browswer = split( m!/!, $user_agent );
    if ( $browswer[0] eq 'DoCoMo' ) {
        if ( $user_agent =~ /^.*(ser[0-9]{11,}).*$/ ) {
            $mobile_id = $1;
        }
    } else {
        my $x_up_subno = $user_agent = $app->get_header( 'X_UP_SUBNO' );
        # AU
        if ( $x_up_subno ) {
            $mobile_id = $x_up_subno;
        } else {
             # Softbank
             my $x_jphone_uid = $app->get_header( 'X_JPHONE_UID' );
             if ( $x_jphone_uid ) {
                 $mobile_id = $x_jphone_uid;
             }
        }
    }
    if ( $mobile_id ) {
        if ( eval { require Digest::MD5; 1; } ) {
            $mobile_id = Digest::MD5::md5_hex( $mobile_id );
        } else {
            $mobile_id = perl_sha1_digest_hex( $mobile_id );
        }
        return $mobile_id if $mobile_id;
    }
    return '';
}

1;